@@ -156,6 +156,11 @@ class method_binder(nested_method_decorator):
 			return f'<future_method {self.fn.__name__} of {self.owner.__name__}>' if self.instance is None \
 				else f'<future_method {self.fn.__name__} of {self.owner.__name__} bound to {self.instance}>'
 
+		# def __get__(self, instance, owner):
+		# 	# if instance is None:
+		# 	# 	return self
+		# 	return self.fn.__get__(instance, owner)
+
 		def __call__(self, *args: Any, **kwargs: Any) -> Any:
 			# if not self.is_static and self.instance is None:
 			# 	assert len(args), 'no instance to call method on'
@@ -329,9 +334,11 @@ class method_wrapper(nested_method_decorator):
 
 
 class auto_methods(capturable_method):
-	# _auto_methods = None
+	_method_capturer = captured_method
+	_auto_wrap_mro_until = None
 	def __init_subclass__(cls, auto_methods: Optional[Union[str, Sequence[str]]] = (), wrap_existing: bool = False,
-	                      inheritable_auto_methods: Optional[Union[str, Sequence[str]]] = (), **kwargs):
+	                      inheritable_auto_methods: Optional[Union[str, Sequence[str]]] = (),
+	                      wrap_mro_until=None, **kwargs):
 		super().__init_subclass__(**kwargs)
 
 		existing = [method for method, inherit in getattr(cls, '_auto_methods', {}).items() if inherit]
@@ -344,8 +351,22 @@ class auto_methods(capturable_method):
 
 		for method in cls._auto_methods:
 			if wrap_existing or method in cls.__dict__:
-				setattr(cls, method, captured_method(inspect.getattr_static(cls, method)).setup(cls, method))
-
+				fn = inspect.getattr_static(cls, method)
+				if not isinstance(fn, cls._method_capturer):
+					setattr(cls, method, cls._method_capturer(fn).setup(cls, method))
+		if wrap_mro_until is not None:
+			if wrap_mro_until is True:
+				wrap_mro_until = cls
+			cls._auto_wrap_mro_until = wrap_mro_until
+		if cls._auto_wrap_mro_until is not None and issubclass(cls, cls._auto_wrap_mro_until):
+			for parent in cls.mro():
+				if parent is cls._auto_wrap_mro_until:
+					break
+				if parent is not cls:
+					for method in cls._auto_methods:
+						fn = parent.__dict__.get(method, None)
+						if fn is not None and not isinstance(fn, cls._method_capturer):
+							setattr(parent, method, cls._method_capturer(fn).setup(parent, method))
 
 	@classmethod
 	def _auto_method_call(cls, self, src: Type, method: Callable, args: Tuple, kwargs: Dict[str, Any]) -> None:
@@ -354,7 +375,7 @@ class auto_methods(capturable_method):
 	@classmethod
 	def captured_method_call(cls, self, src: Type['auto_methods'], fn: Callable,
 	                         args: Tuple, kwargs: Dict[str, Any]) -> Any:
-		if getattr(src, '_auto_methods', None) is not None and fn.__name__ in src._auto_methods:
+		if getattr(cls, '_auto_methods', None) is not None and getattr(fn, '__name__', None) in cls._auto_methods:
 			return cls._auto_method_call(self, src, fn, args, kwargs)
 		return super().captured_method_call(self, src, fn, args, kwargs)
 
@@ -464,6 +485,32 @@ class simple_dynamic_capture(dynamic_capture):
 
 
 
+class ClassHierarchy:
+	_class_hierarchy_root = None
+	_class_hierarchy_parent_ref = None
+	_class_hierarchy_parent = None
+	_class_hierarchy_address = ()
+	_class_hierarchy_children = None
+
+	def __init_subclass__(cls, ident=None, as_branch=False, new_hierarchy=None, **kwargs):
+		super().__init_subclass__(**kwargs)
+		if new_hierarchy is None:
+			new_hierarchy = cls._class_hierarchy_parent_ref is None
+		if new_hierarchy:
+			cls._class_hierarchy_root = cls
+			cls._class_hierarchy_parent = None
+			cls._class_hierarchy_parent_ref = cls
+			cls._class_hierarchy_children = OrderedDict()
+		else:
+			cls._class_hierarchy_parent = cls._class_hierarchy_parent_ref
+			if ident is not None:
+				cls._class_hierarchy_children[ident] = cls
+			if as_branch:
+				cls._class_hierarchy_parent_ref = cls
+				cls._class_hierarchy_children = OrderedDict()
+			cls._class_hierarchy_address = (*cls._class_hierarchy_address, as_branch)
+
+
 
 class smartproperty:
 	unknown = object()
@@ -474,7 +521,7 @@ class smartproperty:
 		self.src = src
 
 		self.cache = cache
-		self.cached_value = self.unknown
+		self.cached_value = self.unknown # only for the class property
 
 		self.fget = fget
 		self.fset = fset
@@ -597,6 +644,9 @@ class smartproperty:
 			if cache is not None:
 				cache[self.name] = value
 
+	def is_cached(self, base):
+		return self._get_cached_value(base) is not self.unknown
+
 	def _get_cached_value(self, base):
 		if base is self.src:
 			return self.cached_value
@@ -819,14 +869,23 @@ class defaultproperty(smartproperty):
 			return None, default  # no fget provided (optionally can be added with __call__)
 		return fget, default  # fget was specified as keyword argument
 
-	def create_value(self, base, owner=None): # TODO: maybe make thread-safe by using a lock
+
+	def get_value(self, base, owner=None): # TODO: maybe make thread-safe by using a lock
 		try:
-			return super().create_value(base, owner)
+			return super().get_value(base, owner)
 		except self.MissingValueError:
 			if self.default is self.unknown:
 				raise
 			return self.default
 
+	# def create_value(self, base, owner=None): # TODO: maybe make thread-safe by using a lock
+	# 	try:
+	# 		return super().create_value(base, owner)
+	# 	except self.MissingValueError:
+	# 		if self.default is self.unknown:
+	# 			raise
+	# 		return self.default
+
 
 from .utils import split_dict
 
@@ -937,6 +996,12 @@ class Modifiable:
 		'''
 		return inject_modifiers(cls, *mods, name=name)
 
+	# @classmethod
+	# def __new_mod__(cls, mods, *args, **kwargs):
+	# 	if isinstance(mods, str):
+	# 		mods = (mods,)
+	# 	return cls.inject_mods(*mods)(*args, **kwargs)
+
 
 # class old_auto_init(capturable_method):
 # 	def __init_subclass__(cls, **kwargs):
@@ -1124,7 +1189,7 @@ def collect_init_kwargs(typ: Type, default: Any = Parameter.empty, *, end_type:
 def extract_function_signature(fn: Union[Callable, Type],
                                args: Optional[Tuple] = None, kwargs: Optional[Dict[str, Any]] = None, *,
                                default_fn: Callable[[str, Any], Any] = None, include_missing: bool = False,
-                               allow_positional: bool = True) \
+                               allow_positional: bool = True, force_no_positional: bool = False, skip_first=None) \
 		-> Union[Tuple[List[Any], Dict[str, Any], List[inspect.Parameter]],
 		         Tuple[List[Any], Dict[str, Any]],
 		         Dict[str, Any]]:
@@ -1135,9 +1200,13 @@ def extract_function_signature(fn: Union[Callable, Type],
 
 	if isinstance(fn, type):
 		fn = fn.__init__
+		skip_first = skip_first is not False
 
 	params = inspect.signature(fn).parameters
-	
+	if skip_first:
+		params = list(params.values())[1:]
+		params = {param.name: param for param in params}
+
 	arg_idx = 0
 	fixed_args = []
 	fixed_kwargs = {}
@@ -1146,7 +1215,7 @@ def extract_function_signature(fn: Union[Callable, Type],
 
 	for n, p in params.items():
 		if p.kind == p.POSITIONAL_ONLY:
-			if not allow_positional:
+			if not allow_positional or force_no_positional:
 				raise TypeError(f'Function {fn.__name__} has a positional only argument ({n})')
 			if arg_idx < len(args):
 				fixed_args.append(args[arg_idx])
@@ -1162,9 +1231,9 @@ def extract_function_signature(fn: Union[Callable, Type],
 				else:
 					fixed_args.append(val)
 		elif p.kind == p.VAR_POSITIONAL:
-			if not allow_positional:
-				# raise TypeError(f'Function {fn.__name__} has variable positional arguments ({n})')
-				continue
+			if force_no_positional:
+				raise TypeError(f'Function {fn.__name__} has variable positional arguments ({n})')
+				# continue
 			try:
 				if default_fn is None:
 					raise KeyError
@@ -1188,10 +1257,10 @@ def extract_function_signature(fn: Union[Callable, Type],
 			if n in kwargs:
 				fixed_kwargs[n] = kwargs[n]
 			elif p.kind != p.KEYWORD_ONLY and arg_idx < len(args):
-				if allow_positional:
-					fixed_args.append(args[arg_idx])
-				else:
-					fixed_kwargs[n] = args[arg_idx]
+				# if allow_positional: # Note: removing this to prefer keyword arguments over positional arguments
+				# 	fixed_args.append(args[arg_idx])
+				# else:
+				fixed_kwargs[n] = args[arg_idx]
 				arg_idx += 1
 			else:
 				try:
